
#include <Arduino.h>

#define IN1 11 // PB5 (OC1A)
#define IN2 12 // PB6 (OC1B)
#define ENA 8

volatile uint32_t count;
uint32_t max_count;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
uint32_t icr;
uint8_t tccr1a,tccr1b;
uint16_t ocr1a,ocr1b;
bool inter;

void config_impulsions(float period, int32_t nombre, float ra, float rb) {
    /*
        periode : T en microsecondes
        nombre : nombre de cyles (nombre de cycles illimité si nombre==0)
        ra : rapport cyclique de OC1A
        rb : rapport cyclique de OC1B
    */
    count = 0;
    if (nombre >0) {
      max_count = nombre+1;
      inter = true;
    }
    else inter = false;
    tccr1a = 0;
    tccr1b = 0;
    //phase and frequency correct PWM mode, top = ICR1 WGM3:0 = 1,0,0,0
    tccr1b |= (1 << WGM13);
    // COM1A1 = 1, COM1A0 = 0 : clear OC1A on compare match when up counting
    // COM1B1 = 1, COM1B0 = 1 : set OC1B on compare match when up counting
    tccr1a |= (1 << COM1A1) | (1 << COM1B1) | (1 << COM1B0);
    icr = (F_CPU/1000000*period/2);
    int d = 1;
    while ((icr>0xFFFF)&&(d<5)) {
        d++;
        icr = (F_CPU/1000000*period/2/diviseur[d]);
    } 
    tccr1b |= d;
    ocr1a = icr*ra;
    ocr1b = icr*(1.0-rb);
    
}


void start_impulsions() {
  cli();
  count = 0;
  TCCR1A = tccr1a;
  TCCR1B = tccr1b;
  TCNT1 = 0;
  ICR1 = icr;
  OCR1A = ocr1a;
  OCR1B = ocr1b;
  if (inter) TIMSK1 = 1 << TOIE1; // overflow interrupt enable
  sei();
}

ISR(TIMER1_OVF_vect) {
  count++;
  if (count==max_count) {
    TCCR1A = 0;
    PORTB &= ~((1 << PORTB5) | (1 << PORTB6)); // PB5 et PB6 LOW
  }
}

void setup() {
  pinMode(IN1,OUTPUT);
  pinMode(IN2,OUTPUT);
  PORTB &= ~(1 << PORTB5);
  PORTB &= ~(1 << PORTB6);
  pinMode(ENA,OUTPUT);
  digitalWrite(ENA,HIGH);
  config_impulsions(13,0,0.5,0.5);
  start_impulsions();
}

void loop() {
    
}
					   
					   